@@ -22,12 +22,12 @@
 #include <ctype.h>
 
 #undef BUFSIZE
-#define BUFSIZE 1024 * 8
+#define BUFSIZE 1024*8
 
 int do_fp(BIO *out, unsigned char *buf, BIO *bp, int sep, int binout, int xoflen,
-    EVP_PKEY *key, unsigned char *sigin, int siglen,
-    const char *sig_name, const char *md_name,
-    const char *file);
+          EVP_PKEY *key, unsigned char *sigin, int siglen,
+          const char *sig_name, const char *md_name,
+          const char *file);
 static void show_digests(const OBJ_NAME *name, void *bio_);
 
 struct doall_dgst_digests {
@@ -38,74 +38,57 @@ struct doall_dgst_digests {
 typedef enum OPTION_choice {
     OPT_COMMON,
     OPT_LIST,
-    OPT_C,
-    OPT_R,
-    OPT_OUT,
-    OPT_SIGN,
-    OPT_PASSIN,
-    OPT_VERIFY,
-    OPT_PRVERIFY,
-    OPT_SIGNATURE,
-    OPT_KEYFORM,
-    OPT_ENGINE,
-    OPT_ENGINE_IMPL,
-    OPT_HEX,
-    OPT_BINARY,
-    OPT_DEBUG,
-    OPT_FIPS_FINGERPRINT,
-    OPT_HMAC,
-    OPT_MAC,
-    OPT_SIGOPT,
-    OPT_MACOPT,
-    OPT_XOFLEN,
+    OPT_C, OPT_R, OPT_OUT, OPT_SIGN, OPT_PASSIN, OPT_VERIFY,
+    OPT_PRVERIFY, OPT_SIGNATURE, OPT_KEYFORM, OPT_ENGINE, OPT_ENGINE_IMPL,
+    OPT_HEX, OPT_BINARY, OPT_DEBUG, OPT_FIPS_FINGERPRINT,
+    OPT_HMAC, OPT_MAC, OPT_SIGOPT, OPT_MACOPT, OPT_XOFLEN,
     OPT_DIGEST,
-    OPT_R_ENUM,
-    OPT_PROV_ENUM
+    OPT_R_ENUM, OPT_PROV_ENUM
 } OPTION_CHOICE;
 
 const OPTIONS dgst_options[] = {
-    { OPT_HELP_STR, 1, '-', "Usage: %s [options] [file...]\n" },
+    {OPT_HELP_STR, 1, '-', "Usage: %s [options] [file...]\n"},
 
     OPT_SECTION("General"),
-    { "help", OPT_HELP, '-', "Display this summary" },
-    { "list", OPT_LIST, '-', "List digests" },
+    {"help", OPT_HELP, '-', "Display this summary"},
+    {"list", OPT_LIST, '-', "List digests"},
 #ifndef OPENSSL_NO_ENGINE
-    { "engine", OPT_ENGINE, 's', "Use engine e, possibly a hardware device" },
-    { "engine_impl", OPT_ENGINE_IMPL, '-',
-        "Also use engine given by -engine for digest operations" },
+    {"engine", OPT_ENGINE, 's', "Use engine e, possibly a hardware device"},
+    {"engine_impl", OPT_ENGINE_IMPL, '-',
+     "Also use engine given by -engine for digest operations"},
 #endif
-    { "passin", OPT_PASSIN, 's', "Input file pass phrase source" },
+    {"passin", OPT_PASSIN, 's', "Input file pass phrase source"},
 
     OPT_SECTION("Output"),
-    { "c", OPT_C, '-', "Print the digest with separating colons" },
-    { "r", OPT_R, '-', "Print the digest in coreutils format" },
-    { "out", OPT_OUT, '>', "Output to filename rather than stdout" },
-    { "keyform", OPT_KEYFORM, 'f', "Key file format (ENGINE, other values ignored)" },
-    { "hex", OPT_HEX, '-', "Print as hex dump" },
-    { "binary", OPT_BINARY, '-', "Print in binary form" },
-    { "xoflen", OPT_XOFLEN, 'p', "Output length for XOF algorithms. To obtain the maximum security strength set this to 32 (or greater) for SHAKE128, and 64 (or greater) for SHAKE256" },
-    { "d", OPT_DEBUG, '-', "Print debug info" },
-    { "debug", OPT_DEBUG, '-', "Print debug info" },
+    {"c", OPT_C, '-', "Print the digest with separating colons"},
+    {"r", OPT_R, '-', "Print the digest in coreutils format"},
+    {"out", OPT_OUT, '>', "Output to filename rather than stdout"},
+    {"keyform", OPT_KEYFORM, 'f', "Key file format (ENGINE, other values ignored)"},
+    {"hex", OPT_HEX, '-', "Print as hex dump"},
+    {"binary", OPT_BINARY, '-', "Print in binary form"},
+    {"xoflen", OPT_XOFLEN, 'p', "Output length for XOF algorithms. To obtain the maximum security strength set this to 32 (or greater) for SHAKE128, and 64 (or greater) for SHAKE256"},
+    {"d", OPT_DEBUG, '-', "Print debug info"},
+    {"debug", OPT_DEBUG, '-', "Print debug info"},
 
     OPT_SECTION("Signing"),
-    { "sign", OPT_SIGN, 's', "Sign digest using private key" },
-    { "verify", OPT_VERIFY, 's', "Verify a signature using public key" },
-    { "prverify", OPT_PRVERIFY, 's', "Verify a signature using private key" },
-    { "sigopt", OPT_SIGOPT, 's', "Signature parameter in n:v form" },
-    { "signature", OPT_SIGNATURE, '<', "File with signature to verify" },
-    { "hmac", OPT_HMAC, 's', "Create hashed MAC with key" },
-    { "mac", OPT_MAC, 's', "Create MAC (not necessarily HMAC)" },
-    { "macopt", OPT_MACOPT, 's', "MAC algorithm parameters in n:v form or key" },
-    { "", OPT_DIGEST, '-', "Any supported digest" },
-    { "fips-fingerprint", OPT_FIPS_FINGERPRINT, '-',
-        "Compute HMAC with the key used in OpenSSL-FIPS fingerprint" },
+    {"sign", OPT_SIGN, 's', "Sign digest using private key"},
+    {"verify", OPT_VERIFY, 's', "Verify a signature using public key"},
+    {"prverify", OPT_PRVERIFY, 's', "Verify a signature using private key"},
+    {"sigopt", OPT_SIGOPT, 's', "Signature parameter in n:v form"},
+    {"signature", OPT_SIGNATURE, '<', "File with signature to verify"},
+    {"hmac", OPT_HMAC, 's', "Create hashed MAC with key"},
+    {"mac", OPT_MAC, 's', "Create MAC (not necessarily HMAC)"},
+    {"macopt", OPT_MACOPT, 's', "MAC algorithm parameters in n:v form or key"},
+    {"", OPT_DIGEST, '-', "Any supported digest"},
+    {"fips-fingerprint", OPT_FIPS_FINGERPRINT, '-',
+     "Compute HMAC with the key used in OpenSSL-FIPS fingerprint"},
 
     OPT_R_OPTIONS,
     OPT_PROV_OPTIONS,
 
     OPT_PARAMETERS(),
-    { "file", 0, 0, "Files to digest (optional; default is stdin)" },
-    { NULL }
+    {"file", 0, 0, "Files to digest (optional; default is stdin)"},
+    {NULL}
 };
 
 int dgst_main(int argc, char **argv)
@@ -134,12 +117,13 @@ int dgst_main(int argc, char **argv)
     if (md != NULL)
         digestname = argv[0];
 
+    opt_set_unknown_name("digest");
     prog = opt_init(argc, argv, dgst_options);
     while ((o = opt_next()) != OPT_EOF) {
         switch (o) {
         case OPT_EOF:
         case OPT_ERR:
-        opthelp:
+ opthelp:
             BIO_printf(bio_err, "%s: Use -help for summary.\n", prog);
             goto end;
         case OPT_HELP:
@@ -151,7 +135,7 @@ int dgst_main(int argc, char **argv)
             dec.bio = bio_out;
             dec.n = 0;
             OBJ_NAME_do_all_sorted(OBJ_NAME_TYPE_MD_METH,
-                show_digests, &dec);
+                                   show_digests, &dec);
             BIO_printf(bio_out, "\n");
             ret = EXIT_SUCCESS;
             goto end;
@@ -255,7 +239,7 @@ int dgst_main(int argc, char **argv)
 
     if (do_verify && sigfile == NULL) {
         BIO_printf(bio_err,
-            "No signature to verify: use the -signature option\n");
+                   "No signature to verify: use the -signature option\n");
         goto end;
     }
     if (engine_impl)
@@ -347,8 +331,8 @@ int dgst_main(int argc, char **argv)
             digestname = SN_sha256;
         }
         sigkey = EVP_PKEY_new_raw_private_key(EVP_PKEY_HMAC, impl,
-            (unsigned char *)hmac_key,
-            strlen(hmac_key));
+                                              (unsigned char *)hmac_key,
+                                              strlen(hmac_key));
         if (sigkey == NULL)
             goto end;
     }
@@ -365,16 +349,17 @@ int dgst_main(int argc, char **argv)
         if (do_verify)
             if (impl == NULL)
                 res = EVP_DigestVerifyInit_ex(mctx, &pctx, digestname,
-                    app_get0_libctx(),
-                    app_get0_propq(), sigkey, NULL);
+                                              app_get0_libctx(),
+                                              app_get0_propq(), sigkey, NULL);
             else
                 res = EVP_DigestVerifyInit(mctx, &pctx, md, impl, sigkey);
-        else if (impl == NULL)
-            res = EVP_DigestSignInit_ex(mctx, &pctx, digestname,
-                app_get0_libctx(),
-                app_get0_propq(), sigkey, NULL);
         else
-            res = EVP_DigestSignInit(mctx, &pctx, md, impl, sigkey);
+            if (impl == NULL)
+                res = EVP_DigestSignInit_ex(mctx, &pctx, digestname,
+                                            app_get0_libctx(),
+                                            app_get0_propq(), sigkey, NULL);
+            else
+                res = EVP_DigestSignInit(mctx, &pctx, md, impl, sigkey);
         if (res == 0) {
             BIO_printf(bio_err, "Error setting context\n");
             goto end;
@@ -385,7 +370,7 @@ int dgst_main(int argc, char **argv)
 
                 if (pkey_ctrl_string(pctx, sigopt) <= 0) {
                     BIO_printf(bio_err, "Signature parameter error \"%s\"\n",
-                        sigopt);
+                               sigopt);
                     goto end;
                 }
             }
@@ -452,7 +437,7 @@ int dgst_main(int argc, char **argv)
     if (argc == 0) {
         BIO_set_fp(in, stdin, BIO_NOCLOSE);
         ret = do_fp(out, buf, inp, separator, out_bin, xoflen, sigkey, sigbuf,
-            siglen, NULL, md_name, "stdin");
+                    siglen, NULL, md_name, "stdin");
     } else {
         const char *sig_name = NULL;
 
@@ -468,13 +453,13 @@ int dgst_main(int argc, char **argv)
                 continue;
             } else {
                 if (do_fp(out, buf, inp, separator, out_bin, xoflen,
-                        sigkey, sigbuf, siglen, sig_name, md_name, argv[i]))
+                          sigkey, sigbuf, siglen, sig_name, md_name, argv[i]))
                     ret = EXIT_FAILURE;
             }
             (void)BIO_reset(bmd);
         }
     }
-end:
+ end:
     if (ret != EXIT_SUCCESS)
         ERR_print_errors(bio_err);
     OPENSSL_clear_free(buf, BUFSIZE);
@@ -556,13 +541,14 @@ static const char *newline_escape_filename(const char *file, int *backslash)
         e++;
     }
     file_cpy[i] = '\0';
-    return (const char *)file_cpy;
+    return (const char*)file_cpy;
 }
 
+
 int do_fp(BIO *out, unsigned char *buf, BIO *bp, int sep, int binout, int xoflen,
-    EVP_PKEY *key, unsigned char *sigin, int siglen,
-    const char *sig_name, const char *md_name,
-    const char *file)
+          EVP_PKEY *key, unsigned char *sigin, int siglen,
+          const char *sig_name, const char *md_name,
+          const char *file)
 {
     size_t len = BUFSIZE;
     int i, backslash = 0, ret = EXIT_FAILURE;
@@ -665,7 +651,7 @@ int do_fp(BIO *out, unsigned char *buf, BIO *bp, int sep, int binout, int xoflen
     }
 
     ret = EXIT_SUCCESS;
-end:
+ end:
     if (allocated_buf != NULL)
         OPENSSL_clear_free(allocated_buf, len);
 
