@@ -1,5 +1,5 @@
 /*
- * Copyright 1995-2026 The OpenSSL Project Authors. All Rights Reserved.
+ * Copyright 1995-2023 The OpenSSL Project Authors. All Rights Reserved.
  *
  * Licensed under the Apache License 2.0 (the "License").  You may not use
  * this file except in compliance with the License.  You can obtain a copy
@@ -45,7 +45,7 @@ struct X509_pubkey_st {
 static int x509_pubkey_decode(EVP_PKEY **pk, const X509_PUBKEY *key);
 
 static int x509_pubkey_set0_libctx(X509_PUBKEY *x, OSSL_LIB_CTX *libctx,
-    const char *propq)
+                                   const char *propq)
 {
     if (x != NULL) {
         x->libctx = libctx;
@@ -61,20 +61,21 @@ static int x509_pubkey_set0_libctx(X509_PUBKEY *x, OSSL_LIB_CTX *libctx,
 }
 
 ASN1_SEQUENCE(X509_PUBKEY_INTERNAL) = {
-    ASN1_SIMPLE(X509_PUBKEY, algor, X509_ALGOR),
-    ASN1_SIMPLE(X509_PUBKEY, public_key, ASN1_BIT_STRING)
+        ASN1_SIMPLE(X509_PUBKEY, algor, X509_ALGOR),
+        ASN1_SIMPLE(X509_PUBKEY, public_key, ASN1_BIT_STRING)
 } static_ASN1_SEQUENCE_END_name(X509_PUBKEY, X509_PUBKEY_INTERNAL)
 
-                                          X509_PUBKEY
-    * ossl_d2i_X509_PUBKEY_INTERNAL(const unsigned char **pp, long len, OSSL_LIB_CTX *libctx)
+X509_PUBKEY *ossl_d2i_X509_PUBKEY_INTERNAL(const unsigned char **pp,
+                                           long len, OSSL_LIB_CTX *libctx,
+                                           const char *propq)
 {
     X509_PUBKEY *xpub = OPENSSL_zalloc(sizeof(*xpub));
 
     if (xpub == NULL)
         return NULL;
     return (X509_PUBKEY *)ASN1_item_d2i_ex((ASN1_VALUE **)&xpub, pp, len,
-        ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL),
-        libctx, NULL);
+                                           ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL),
+                                           libctx, propq);
 }
 
 void ossl_X509_PUBKEY_INTERNAL_free(X509_PUBKEY *xpub)
@@ -101,22 +102,24 @@ static int x509_pubkey_ex_populate(ASN1_VALUE **pval, const ASN1_ITEM *it)
     X509_PUBKEY *pubkey = (X509_PUBKEY *)*pval;
 
     return (pubkey->algor != NULL
-               || (pubkey->algor = X509_ALGOR_new()) != NULL)
+            || (pubkey->algor = X509_ALGOR_new()) != NULL)
         && (pubkey->public_key != NULL
             || (pubkey->public_key = ASN1_BIT_STRING_new()) != NULL);
 }
 
+
 static int x509_pubkey_ex_new_ex(ASN1_VALUE **pval, const ASN1_ITEM *it,
-    OSSL_LIB_CTX *libctx, const char *propq)
+                                 OSSL_LIB_CTX *libctx, const char *propq)
 {
     X509_PUBKEY *ret;
 
-    if ((ret = OPENSSL_zalloc(sizeof(*ret))) == NULL
-        || !x509_pubkey_ex_populate((ASN1_VALUE **)&ret, NULL)
+    if ((ret = OPENSSL_zalloc(sizeof(*ret))) == NULL)
+        return 0;
+    if (!x509_pubkey_ex_populate((ASN1_VALUE **)&ret, NULL)
         || !x509_pubkey_set0_libctx(ret, libctx, propq)) {
         x509_pubkey_ex_free((ASN1_VALUE **)&ret, NULL);
         ret = NULL;
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_X509_LIB);
     } else {
         *pval = (ASN1_VALUE *)ret;
     }
@@ -125,10 +128,10 @@ static int x509_pubkey_ex_new_ex(ASN1_VALUE **pval, const ASN1_ITEM *it,
 }
 
 static int x509_pubkey_ex_d2i_ex(ASN1_VALUE **pval,
-    const unsigned char **in, long len,
-    const ASN1_ITEM *it, int tag, int aclass,
-    char opt, ASN1_TLC *ctx, OSSL_LIB_CTX *libctx,
-    const char *propq)
+                                 const unsigned char **in, long len,
+                                 const ASN1_ITEM *it, int tag, int aclass,
+                                 char opt, ASN1_TLC *ctx, OSSL_LIB_CTX *libctx,
+                                 const char *propq)
 {
     const unsigned char *in_saved = *in;
     size_t publen;
@@ -140,19 +143,15 @@ static int x509_pubkey_ex_d2i_ex(ASN1_VALUE **pval,
     if (*pval == NULL && !x509_pubkey_ex_new_ex(pval, it, libctx, propq))
         return 0;
     if (!x509_pubkey_ex_populate(pval, NULL)) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_X509_LIB);
         return 0;
     }
 
     /* This ensures that |*in| advances properly no matter what */
-    if ((ret = asn1_item_embed_d2i(pval, in, len,
-             ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL),
-             tag, aclass, opt, ctx, 0,
-             NULL, NULL))
-        <= 0) {
-        x509_pubkey_ex_free(pval, it);
+    if ((ret = ASN1_item_ex_d2i(pval, in, len,
+                                ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL),
+                                tag, aclass, opt, ctx)) <= 0)
         return ret;
-    }
 
     publen = *in - in_saved;
     if (!ossl_assert(publen > 0)) {
@@ -188,32 +187,29 @@ static int x509_pubkey_ex_d2i_ex(ASN1_VALUE **pval,
         size_t slen = publen;
 
         /*
-         * The decoders don't know how to handle anything other than Universal
-         * class so we modify the data accordingly.
-         */
+        * The decoders don't know how to handle anything other than Universal
+        * class so we modify the data accordingly.
+        */
         if (aclass != V_ASN1_UNIVERSAL) {
             tmpbuf = OPENSSL_memdup(in_saved, publen);
-            if (tmpbuf == NULL) {
-                ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+            if (tmpbuf == NULL)
                 return 0;
-            }
             in_saved = tmpbuf;
             *tmpbuf = V_ASN1_CONSTRUCTED | V_ASN1_SEQUENCE;
         }
         p = in_saved;
 
         if (OBJ_obj2txt(txtoidname, sizeof(txtoidname),
-                pubkey->algor->algorithm, 0)
-            <= 0) {
+                        pubkey->algor->algorithm, 0) <= 0) {
             ERR_clear_last_mark();
             goto end;
         }
-        if ((dctx = OSSL_DECODER_CTX_new_for_pkey(&pubkey->pkey,
-                 "DER", "SubjectPublicKeyInfo",
-                 txtoidname, EVP_PKEY_PUBLIC_KEY,
-                 pubkey->libctx,
-                 pubkey->propq))
-            != NULL)
+        if ((dctx =
+             OSSL_DECODER_CTX_new_for_pkey(&pubkey->pkey,
+                                           "DER", "SubjectPublicKeyInfo",
+                                           txtoidname, EVP_PKEY_PUBLIC_KEY,
+                                           pubkey->libctx,
+                                           pubkey->propq)) != NULL)
             /*
              * As said higher up, we're being opportunistic.  In other words,
              * we don't care if we fail.
@@ -233,31 +229,31 @@ static int x509_pubkey_ex_d2i_ex(ASN1_VALUE **pval,
 
     ERR_pop_to_mark();
     ret = 1;
-end:
+ end:
     OSSL_DECODER_CTX_free(dctx);
     OPENSSL_free(tmpbuf);
     return ret;
 }
 
 static int x509_pubkey_ex_i2d(const ASN1_VALUE **pval, unsigned char **out,
-    const ASN1_ITEM *it, int tag, int aclass)
+                              const ASN1_ITEM *it, int tag, int aclass)
 {
     return ASN1_item_ex_i2d(pval, out, ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL),
-        tag, aclass);
+                            tag, aclass);
 }
 
 static int x509_pubkey_ex_print(BIO *out, const ASN1_VALUE **pval, int indent,
-    const char *fname, const ASN1_PCTX *pctx)
+                                const char *fname, const ASN1_PCTX *pctx)
 {
     return ASN1_item_print(out, *pval, indent,
-        ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL), pctx);
+                           ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL), pctx);
 }
 
 static const ASN1_EXTERN_FUNCS x509_pubkey_ff = {
     NULL,
     NULL,
     x509_pubkey_ex_free,
-    0, /* Default clear behaviour is OK */
+    0,                          /* Default clear behaviour is OK */
     NULL,
     x509_pubkey_ex_i2d,
     x509_pubkey_ex_print,
@@ -288,16 +284,22 @@ X509_PUBKEY *X509_PUBKEY_dup(const X509_PUBKEY *a)
 {
     X509_PUBKEY *pubkey = OPENSSL_zalloc(sizeof(*pubkey));
 
-    if (pubkey == NULL
-        || !x509_pubkey_set0_libctx(pubkey, a->libctx, a->propq)
-        || (pubkey->algor = X509_ALGOR_dup(a->algor)) == NULL
+    if (pubkey == NULL)
+        return NULL;
+    if (!x509_pubkey_set0_libctx(pubkey, a->libctx, a->propq)) {
+        ERR_raise(ERR_LIB_X509, ERR_R_X509_LIB);
+        x509_pubkey_ex_free((ASN1_VALUE **)&pubkey,
+                            ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL));
+        return NULL;
+    }
+    if ((pubkey->algor = X509_ALGOR_dup(a->algor)) == NULL
         || (pubkey->public_key = ASN1_BIT_STRING_new()) == NULL
         || !ASN1_BIT_STRING_set(pubkey->public_key,
-            a->public_key->data,
-            a->public_key->length)) {
+                                a->public_key->data,
+                                a->public_key->length)) {
         x509_pubkey_ex_free((ASN1_VALUE **)&pubkey,
-            ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL));
-        ERR_raise(ERR_LIB_X509, ERR_R_MALLOC_FAILURE);
+                            ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL));
+        ERR_raise(ERR_LIB_X509, ERR_R_ASN1_LIB);
         return NULL;
     }
 
@@ -308,7 +310,7 @@ X509_PUBKEY *X509_PUBKEY_dup(const X509_PUBKEY *a)
             pubkey->flag_force_legacy = 1;
             if (x509_pubkey_decode(&pubkey->pkey, pubkey) <= 0) {
                 x509_pubkey_ex_free((ASN1_VALUE **)&pubkey,
-                    ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL));
+                                    ASN1_ITEM_rptr(X509_PUBKEY_INTERNAL));
                 ERR_clear_last_mark();
                 return NULL;
             }
@@ -329,7 +331,7 @@ int X509_PUBKEY_set(X509_PUBKEY **x, EVP_PKEY *pkey)
 
     if (pkey->ameth != NULL) {
         if ((pk = X509_PUBKEY_new()) == NULL) {
-            ERR_raise(ERR_LIB_X509, ERR_R_MALLOC_FAILURE);
+            ERR_raise(ERR_LIB_X509, ERR_R_ASN1_LIB);
             goto error;
         }
         if (pkey->ameth->pub_encode != NULL) {
@@ -344,9 +346,10 @@ int X509_PUBKEY_set(X509_PUBKEY **x, EVP_PKEY *pkey)
     } else if (evp_pkey_is_provided(pkey)) {
         unsigned char *der = NULL;
         size_t derlen = 0;
-        OSSL_ENCODER_CTX *ectx = OSSL_ENCODER_CTX_new_for_pkey(pkey, EVP_PKEY_PUBLIC_KEY,
-            "DER", "SubjectPublicKeyInfo",
-            NULL);
+        OSSL_ENCODER_CTX *ectx =
+            OSSL_ENCODER_CTX_new_for_pkey(pkey, EVP_PKEY_PUBLIC_KEY,
+                                          "DER", "SubjectPublicKeyInfo",
+                                          NULL);
 
         if (OSSL_ENCODER_to_data(ectx, &der, &derlen)) {
             const unsigned char *pder = der;
@@ -386,7 +389,7 @@ int X509_PUBKEY_set(X509_PUBKEY **x, EVP_PKEY *pkey)
     pk->pkey = pkey;
     return 1;
 
-error:
+ error:
     X509_PUBKEY_free(pk);
     return 0;
 }
@@ -419,7 +422,7 @@ static int x509_pubkey_decode(EVP_PKEY **ppkey, const X509_PUBKEY *key)
 
     pkey = EVP_PKEY_new();
     if (pkey == NULL) {
-        ERR_raise(ERR_LIB_X509, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_X509, ERR_R_EVP_LIB);
         return -1;
     }
 
@@ -444,7 +447,7 @@ static int x509_pubkey_decode(EVP_PKEY **ppkey, const X509_PUBKEY *key)
     *ppkey = pkey;
     return 1;
 
-error:
+ error:
     EVP_PKEY_free(pkey);
     return 0;
 }
@@ -481,12 +484,13 @@ EVP_PKEY *X509_PUBKEY_get(const X509_PUBKEY *key)
  * or decode as X509_PUBKEY
  */
 static EVP_PKEY *d2i_PUBKEY_int(EVP_PKEY **a,
-    const unsigned char **pp, long length,
-    OSSL_LIB_CTX *libctx, const char *propq,
-    unsigned int force_legacy,
-    X509_PUBKEY *(*d2i_x509_pubkey)(X509_PUBKEY **a,
-        const unsigned char **in,
-        long len))
+                                const unsigned char **pp, long length,
+                                OSSL_LIB_CTX *libctx, const char *propq,
+                                unsigned int force_legacy,
+                                X509_PUBKEY *
+                                (*d2i_x509_pubkey)(X509_PUBKEY **a,
+                                                   const unsigned char **in,
+                                                   long len))
 {
     X509_PUBKEY *xpk, *xpk2 = NULL, **pxpk = NULL;
     EVP_PKEY *pktmp = NULL;
@@ -501,10 +505,8 @@ static EVP_PKEY *d2i_PUBKEY_int(EVP_PKEY **a,
      */
     if (libctx != NULL || propq != NULL || force_legacy) {
         xpk2 = OPENSSL_zalloc(sizeof(*xpk2));
-        if (xpk2 == NULL) {
-            ERR_raise(ERR_LIB_X509, ERR_R_MALLOC_FAILURE);
+        if (xpk2 == NULL)
             return NULL;
-        }
         if (!x509_pubkey_set0_libctx(xpk2, libctx, propq))
             goto end;
         xpk2->flag_force_legacy = !!force_legacy;
@@ -515,7 +517,7 @@ static EVP_PKEY *d2i_PUBKEY_int(EVP_PKEY **a,
         goto end;
     pktmp = X509_PUBKEY_get(xpk);
     X509_PUBKEY_free(xpk);
-    xpk2 = NULL; /* We know that xpk == xpk2 */
+    xpk2 = NULL;                 /* We know that xpk == xpk2 */
     if (pktmp == NULL)
         goto end;
     *pp = q;
@@ -523,20 +525,20 @@ static EVP_PKEY *d2i_PUBKEY_int(EVP_PKEY **a,
         EVP_PKEY_free(*a);
         *a = pktmp;
     }
-end:
+ end:
     X509_PUBKEY_free(xpk2);
     return pktmp;
 }
 
 /* For the algorithm specific d2i functions further down */
 EVP_PKEY *ossl_d2i_PUBKEY_legacy(EVP_PKEY **a, const unsigned char **pp,
-    long length)
+                                 long length)
 {
     return d2i_PUBKEY_int(a, pp, length, NULL, NULL, 1, d2i_X509_PUBKEY);
 }
 
 EVP_PKEY *d2i_PUBKEY_ex(EVP_PKEY **a, const unsigned char **pp, long length,
-    OSSL_LIB_CTX *libctx, const char *propq)
+                        OSSL_LIB_CTX *libctx, const char *propq)
 {
     return d2i_PUBKEY_int(a, pp, length, libctx, propq, 0, d2i_X509_PUBKEY);
 }
@@ -566,9 +568,10 @@ int i2d_PUBKEY(const EVP_PKEY *a, unsigned char **pp)
         }
         X509_PUBKEY_free(xpk);
     } else if (a->keymgmt != NULL) {
-        OSSL_ENCODER_CTX *ctx = OSSL_ENCODER_CTX_new_for_pkey(a, EVP_PKEY_PUBLIC_KEY,
-            "DER", "SubjectPublicKeyInfo",
-            NULL);
+        OSSL_ENCODER_CTX *ctx =
+            OSSL_ENCODER_CTX_new_for_pkey(a, EVP_PKEY_PUBLIC_KEY,
+                                          "DER", "SubjectPublicKeyInfo",
+                                          NULL);
         BIO *out = BIO_new(BIO_s_mem());
         BUF_MEM *buf = NULL;
 
@@ -629,7 +632,7 @@ int i2d_RSA_PUBKEY(const RSA *a, unsigned char **pp)
         return 0;
     pktmp = EVP_PKEY_new();
     if (pktmp == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign_RSA(pktmp, (RSA *)a);
@@ -671,7 +674,7 @@ int ossl_i2d_DH_PUBKEY(const DH *a, unsigned char **pp)
         return 0;
     pktmp = EVP_PKEY_new();
     if (pktmp == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign_DH(pktmp, (DH *)a);
@@ -712,7 +715,7 @@ int ossl_i2d_DHx_PUBKEY(const DH *a, unsigned char **pp)
         return 0;
     pktmp = EVP_PKEY_new();
     if (pktmp == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign(pktmp, EVP_PKEY_DHX, (DH *)a);
@@ -778,7 +781,7 @@ int i2d_DSA_PUBKEY(const DSA *a, unsigned char **pp)
         return 0;
     pktmp = EVP_PKEY_new();
     if (pktmp == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign_DSA(pktmp, (DSA *)a);
@@ -823,7 +826,7 @@ int i2d_EC_PUBKEY(const EC_KEY *a, unsigned char **pp)
     if (a == NULL)
         return 0;
     if ((pktmp = EVP_PKEY_new()) == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign_EC_KEY(pktmp, (EC_KEY *)a);
@@ -833,8 +836,9 @@ int i2d_EC_PUBKEY(const EC_KEY *a, unsigned char **pp)
     return ret;
 }
 
+# ifndef OPENSSL_NO_ECX
 ECX_KEY *ossl_d2i_ED25519_PUBKEY(ECX_KEY **a,
-    const unsigned char **pp, long length)
+                                 const unsigned char **pp, long length)
 {
     EVP_PKEY *pkey;
     ECX_KEY *key = NULL;
@@ -864,7 +868,7 @@ int ossl_i2d_ED25519_PUBKEY(const ECX_KEY *a, unsigned char **pp)
     if (a == NULL)
         return 0;
     if ((pktmp = EVP_PKEY_new()) == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign(pktmp, EVP_PKEY_ED25519, (ECX_KEY *)a);
@@ -875,7 +879,7 @@ int ossl_i2d_ED25519_PUBKEY(const ECX_KEY *a, unsigned char **pp)
 }
 
 ECX_KEY *ossl_d2i_ED448_PUBKEY(ECX_KEY **a,
-    const unsigned char **pp, long length)
+                               const unsigned char **pp, long length)
 {
     EVP_PKEY *pkey;
     ECX_KEY *key = NULL;
@@ -906,7 +910,7 @@ int ossl_i2d_ED448_PUBKEY(const ECX_KEY *a, unsigned char **pp)
     if (a == NULL)
         return 0;
     if ((pktmp = EVP_PKEY_new()) == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign(pktmp, EVP_PKEY_ED448, (ECX_KEY *)a);
@@ -917,7 +921,7 @@ int ossl_i2d_ED448_PUBKEY(const ECX_KEY *a, unsigned char **pp)
 }
 
 ECX_KEY *ossl_d2i_X25519_PUBKEY(ECX_KEY **a,
-    const unsigned char **pp, long length)
+                                const unsigned char **pp, long length)
 {
     EVP_PKEY *pkey;
     ECX_KEY *key = NULL;
@@ -948,7 +952,7 @@ int ossl_i2d_X25519_PUBKEY(const ECX_KEY *a, unsigned char **pp)
     if (a == NULL)
         return 0;
     if ((pktmp = EVP_PKEY_new()) == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign(pktmp, EVP_PKEY_X25519, (ECX_KEY *)a);
@@ -959,7 +963,7 @@ int ossl_i2d_X25519_PUBKEY(const ECX_KEY *a, unsigned char **pp)
 }
 
 ECX_KEY *ossl_d2i_X448_PUBKEY(ECX_KEY **a,
-    const unsigned char **pp, long length)
+                              const unsigned char **pp, long length)
 {
     EVP_PKEY *pkey;
     ECX_KEY *key = NULL;
@@ -990,7 +994,7 @@ int ossl_i2d_X448_PUBKEY(const ECX_KEY *a, unsigned char **pp)
     if (a == NULL)
         return 0;
     if ((pktmp = EVP_PKEY_new()) == NULL) {
-        ERR_raise(ERR_LIB_ASN1, ERR_R_MALLOC_FAILURE);
+        ERR_raise(ERR_LIB_ASN1, ERR_R_EVP_LIB);
         return -1;
     }
     (void)EVP_PKEY_assign(pktmp, EVP_PKEY_X448, (ECX_KEY *)a);
@@ -1000,28 +1004,30 @@ int ossl_i2d_X448_PUBKEY(const ECX_KEY *a, unsigned char **pp)
     return ret;
 }
 
+# endif /* OPENSSL_NO_ECX */ 
 #endif
 
+void X509_PUBKEY_set0_public_key(X509_PUBKEY *pub,
+                                 unsigned char *penc, int penclen)
+{
+    ASN1_STRING_set0(pub->public_key, penc, penclen);
+    ossl_asn1_string_set_bits_left(pub->public_key, 0);
+}
+
 int X509_PUBKEY_set0_param(X509_PUBKEY *pub, ASN1_OBJECT *aobj,
-    int ptype, void *pval,
-    unsigned char *penc, int penclen)
+                           int ptype, void *pval,
+                           unsigned char *penc, int penclen)
 {
     if (!X509_ALGOR_set0(pub->algor, aobj, ptype, pval))
         return 0;
-    if (penc) {
-        OPENSSL_free(pub->public_key->data);
-        pub->public_key->data = penc;
-        pub->public_key->length = penclen;
-        /* Set number of unused bits to zero */
-        pub->public_key->flags &= ~(ASN1_STRING_FLAG_BITS_LEFT | 0x07);
-        pub->public_key->flags |= ASN1_STRING_FLAG_BITS_LEFT;
-    }
+    if (penc != NULL)
+        X509_PUBKEY_set0_public_key(pub, penc, penclen);
     return 1;
 }
 
 int X509_PUBKEY_get0_param(ASN1_OBJECT **ppkalg,
-    const unsigned char **pk, int *ppklen,
-    X509_ALGOR **pa, const X509_PUBKEY *pub)
+                           const unsigned char **pk, int *ppklen,
+                           X509_ALGOR **pa, const X509_PUBKEY *pub)
 {
     if (ppkalg)
         *ppkalg = pub->algor->algorithm;
@@ -1063,7 +1069,7 @@ int X509_PUBKEY_eq(const X509_PUBKEY *a, const X509_PUBKEY *b)
 }
 
 int ossl_x509_PUBKEY_get0_libctx(OSSL_LIB_CTX **plibctx, const char **ppropq,
-    const X509_PUBKEY *key)
+                                 const X509_PUBKEY *key)
 {
     if (plibctx)
         *plibctx = key->libctx;
