@@ -1,6 +1,6 @@
 #! /usr/bin/env perl
 #
-# Copyright 2020-2022 The OpenSSL Project Authors. All Rights Reserved.
+# Copyright 2020-2024 The OpenSSL Project Authors. All Rights Reserved.
 # Copyright Siemens AG 2019-2022
 #
 # Licensed under the Apache License 2.0 (the "License").
@@ -131,14 +131,29 @@ while ($ARGV[0] =~ m/^-(\w|-[\w\-]+)$/) {
     }
 }
 
-# status variables
+# state variables
 my $self_test;             # whether the current input file is regarded to contain (positive/negative) self-tests
+
+my $in_comment;            # number of lines so far within multi-line comment, 0 if no comment, < 0 when end is on current line
+my $leading_comment;       # multi-line comment has no code before its beginning delimiter, if $in_comment != 0
+my $formatted_comment;     # multi-line comment beginning with "/*-", which indicates/allows special formatting, if $in_comment != 0
+my $comment_indent;        # comment indent, if $in_comment != 0
+
+my $ifdef__cplusplus;      # line before contained '#ifdef __cplusplus' (used in header files)
+my $preproc_if_nesting;    # currently required indentation of preprocessor directive according to #if(n)(def)
+my $in_preproc;            # 0 or number of lines so far within preprocessor directive, e.g., macro definition
+my $preproc_directive;     # name of current preprocessor directive, if $in_preproc != 0
+my $preproc_offset;        # offset to $block_indent within multi-line preprocessor directive, else 0
+my $in_macro_header;       # number of open parentheses + 1 in (multi-line) header of #define, if $in_preproc != 0
+
 my $line;                  # current line number
 my $line_before;           # number of previous not essentially blank line (containing at most whitespace and '\')
 my $line_before2;          # number of not essentially blank line before previous not essentially blank line
+
+# indentation state
 my $contents;              # contents of current line (without blinding)
 #  $_                      # current line, where comments etc. get blinded
-my $code_contents_before;  # contents of previous non-comment non-directive line (without blinding), initially ""
+my $code_contents_before;  # contents of previous non-comment non-preprocessor-directive line (without blinding), initially ""
 my $contents_before;       # contents of $line_before (without blinding), if $line_before > 0
 my $contents_before_;      # contents of $line_before after blinding comments etc., if $line_before > 0
 my $contents_before2;      # contents of $line_before2  (without blinding), if $line_before2 > 0
@@ -150,12 +165,11 @@ my $count_before;          # number of leading whitespace characters (except lin
 my $has_label;             # current line contains label
 my $local_offset;          # current extra indent due to label, switch case/default, or leading closing brace(s)
 my $line_body_start;       # number of line where last function body started, or 0
-my $line_function_start;   # number of line where last function definition started, used if $line_body_start != 0
-my $last_function_header;  # header containing name of last function defined, used if $line_function_start != 0
-my $line_opening_brace;    # number of previous line with opening brace after do/while/for, optionally for if/else
+my $line_function_start;   # number of line where last function definition started, used for $line_body_start
+my $last_function_header;  # header containing name of last function defined, used if $line_body_start != 0
+my $line_opening_brace;    # number of previous line with opening brace after if/do/while/for, optionally for 'else'
 
 my $keyword_opening_brace; # name of previous keyword, used if $line_opening_brace != 0
-my $ifdef__cplusplus;      # line before contained '#ifdef __cplusplus' (used in header files)
 my $block_indent;          # currently required normal indentation at block/statement level
 my $hanging_offset;        # extra indent, which may be nested, for just one hanging statement or expr or typedef
 my @in_do_hanging_offsets; # stack of hanging offsets for nested 'do' ... 'while'
@@ -173,26 +187,26 @@ my $in_block_decls;        # number of local declaration lines after block openi
 my $in_expr;               # in expression after if/while/for/switch/return/enum/LHS of assignment
 my $in_paren_expr;         # in parenthesized if/while/for condition and switch expression, if $expr_indent != 0
 my $in_typedecl;           # nesting level of typedef/struct/union/enum
-my $in_directive;          # number of lines so far within preprocessor directive, e.g., macro definition
-my $directive_nesting;     # currently required indentation of preprocessor directive according to #if(n)(def)
-my $directive_offset;      # indent offset within multi-line preprocessor directive, if $in_directive > 0
-my $in_macro_header;       # number of open parentheses + 1 in (multi-line) header of #define, if $in_directive > 0
-my $in_comment;            # number of lines so far within multi-line comment, or < 0 when end is on current line
-my $leading_comment;       # multi-line comment has no code before its beginning delimiter
-my $formatted_comment;     # multi-line comment beginning with "/*-", which indicates/allows special formatting
-my $comment_indent;        # comment indent, if $in_comment != 0
+
 my $num_reports_line = 0;  # number of issues found on current line
 my $num_reports = 0;       # total number of issues found
 my $num_indent_reports = 0;# total number of indentation issues found
-my $num_nesting_issues = 0;# total number of directive nesting issues found
+my $num_nesting_issues = 0;# total number of preprocessor #if nesting issues found
 my $num_syntax_issues = 0; # total number of syntax issues found during sanity checks
 my $num_SPC_reports = 0;   # total number of whitespace issues found
 my $num_length_reports = 0;# total number of line length issues found
 
 sub reset_file_state {
+    $in_comment = 0;
+    $ifdef__cplusplus = 0;
+    $preproc_if_nesting = 0;
+    $in_preproc = 0;
     $line = 0;
     $line_before = 0;
     $line_before2 = 0;
+    reset_indentation_state();
+}
+sub reset_indentation_state {
     $code_contents_before = "";
     @nested_block_indents = ();
     @nested_hanging_offsets = ();
@@ -209,14 +223,76 @@ sub reset_file_state {
     @in_if_hanging_offsets = ();
     $if_maybe_terminated = 0;
     $block_indent = 0;
-    $ifdef__cplusplus = 0;
     $in_multiline_string = 0;
     $line_body_start = 0;
     $line_opening_brace = 0;
     $in_typedecl = 0;
-    $in_directive = 0;
-    $directive_nesting = 0;
-    $in_comment = 0;
+}
+my $bak_line_before;
+my $bak_line_before2;
+my $bak_code_contents_before;
+my @bak_nested_block_indents;
+my @bak_nested_hanging_offsets;
+my @bak_nested_in_typedecl;
+my @bak_nested_symbols;
+my @bak_nested_indents;
+my @bak_nested_conds_indents;
+my $bak_expr_indent;
+my $bak_in_block_decls;
+my $bak_in_expr;
+my $bak_in_paren_expr;
+my $bak_hanging_offset;
+my @bak_in_do_hanging_offsets;
+my @bak_in_if_hanging_offsets;
+my $bak_if_maybe_terminated;
+my $bak_block_indent;
+my $bak_in_multiline_string;
+my $bak_line_body_start;
+my $bak_line_opening_brace;
+my $bak_in_typedecl;
+sub backup_indentation_state {
+    $bak_code_contents_before = $code_contents_before;
+    @bak_nested_block_indents = @nested_block_indents;
+    @bak_nested_hanging_offsets = @nested_hanging_offsets;
+    @bak_nested_in_typedecl = @nested_in_typedecl;
+    @bak_nested_symbols = @nested_symbols;
+    @bak_nested_indents = @nested_indents;
+    @bak_nested_conds_indents = @nested_conds_indents;
+    $bak_expr_indent = $expr_indent;
+    $bak_in_block_decls = $in_block_decls;
+    $bak_in_expr = $in_expr;
+    $bak_in_paren_expr = $in_paren_expr;
+    $bak_hanging_offset = $hanging_offset;
+    @bak_in_do_hanging_offsets = @in_do_hanging_offsets;
+    @bak_in_if_hanging_offsets = @in_if_hanging_offsets;
+    $bak_if_maybe_terminated = $if_maybe_terminated;
+    $bak_block_indent = $block_indent;
+    $bak_in_multiline_string = $in_multiline_string;
+    $bak_line_body_start = $line_body_start;
+    $bak_line_opening_brace = $line_opening_brace;
+    $bak_in_typedecl = $in_typedecl;
+}
+sub restore_indentation_state {
+    $code_contents_before = $bak_code_contents_before;
+    @nested_block_indents = @bak_nested_block_indents;
+    @nested_hanging_offsets = @bak_nested_hanging_offsets;
+    @nested_in_typedecl = @bak_nested_in_typedecl;
+    @nested_symbols = @bak_nested_symbols;
+    @nested_indents = @bak_nested_indents;
+    @nested_conds_indents = @bak_nested_conds_indents;
+    $expr_indent = $bak_expr_indent;
+    $in_block_decls = $bak_in_block_decls;
+    $in_expr = $bak_in_expr;
+    $in_paren_expr = $bak_in_paren_expr;
+    $hanging_offset = $bak_hanging_offset;
+    @in_do_hanging_offsets = @bak_in_do_hanging_offsets;
+    @in_if_hanging_offsets = @bak_in_if_hanging_offsets;
+    $if_maybe_terminated = $bak_if_maybe_terminated;
+    $block_indent = $bak_block_indent;
+    $in_multiline_string = $bak_in_multiline_string;
+    $line_body_start = $bak_line_body_start;
+    $line_opening_brace = $bak_line_opening_brace;
+    $in_typedecl = $bak_in_typedecl;
 }
 
 # auxiliary submodules @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@ -225,14 +301,14 @@ sub report_flexibly {
     my $line = shift;
     my $msg = shift;
     my $contents = shift;
-    my $report_SPC = $msg =~ /space/;
+    my $report_SPC = $msg =~ /space|blank/;
     return if $report_SPC && $sloppy_SPC;
 
     print "$ARGV:$line:$msg:$contents" unless $self_test;
     $num_reports_line++;
     $num_reports++;
-    $num_indent_reports++ if $msg =~ m/indent/;
-    $num_nesting_issues++ if $msg =~ m/directive nesting/;
+    $num_indent_reports++ if $msg =~ m/:indent /;
+    $num_nesting_issues++ if $msg =~ m/ nesting indent /;
     $num_syntax_issues++  if $msg =~ m/unclosed|unexpected/;
     $num_SPC_reports++    if $report_SPC;
     $num_length_reports++ if $msg =~ m/length/;
@@ -260,6 +336,7 @@ sub blind_nonspace { # blind non-space text of comment as @, preserving length a
 
 sub check_indent { # used for lines outside multi-line string literals
     my $stmt_indent = $block_indent + $hanging_offset + $local_offset;
+    # print "DEBUG: expr_indent $expr_indent; stmt_indent $stmt_indent = block_indent $block_indent + hanging_offset $hanging_offset + local_offset $local_offset\n";
     $stmt_indent = 0 if $stmt_indent < 0; # TODO maybe give warning/error
     my $stmt_desc = $contents =~
         m/^\s*\/\*/ ? "intra-line comment" :
@@ -317,7 +394,7 @@ sub check_indent { # used for lines outside multi-line string literals
         && $line_before > 0 # there is a line before
         && $contents_before_ =~ m/^(\s*)@[\s@]*$/) { # line before begins with '@', no code follows (except '\')
         report_flexibly($line_before, "entire-line comment indent = $count_before != $count (of following line)",
-            $contents_before) if !$sloppy_cmt && $count_before != $count;
+            $contents_before) if !$sloppy_cmt && $count_before != -1 && $count_before != $count;
     }
     # ... but allow normal indentation for the current line, else above check will be done for the line before
     if (($in_comment == 0 || $in_comment < 0) # (no comment,) intra-line comment or end of multi-line comment
@@ -375,7 +452,7 @@ sub update_nested_indents { # may reset $in_paren_expr and in this case also res
         my ($head, $tail) = (substr($str, 0, $i).$1, $3);
         $i += length($1) + length($2) - 1;
 
-        # stop at terminator outside 'for(..;..;..)', assuming that 'for' is followed by '('
+        # stop at terminator outside 'for (..;..;..)', assuming that 'for' is followed by '('
         return $i if $c eq ";" && (!$in_paren_expr || @nested_indents == 0);
 
         my $in_stmt = $in_expr || @nested_symbols != 0; # not: || $in_typedecl != 0
@@ -509,9 +586,9 @@ while (<>) { # loop over all lines of all input files
         s#^(([^"]*"[^"]*")*[^"]*)("[^"]*)\\(\s*)$#$1.($3 =~ tr/"/@/cr).'"'.$4#e;
         # its contents have been blinded and the trailing '\' replaced by '"'
 
-    # strip any other trailing '\' along with any whitespace around it such that it does not interfere with various
-    # matching below; the later handling of multi-line macro definitions uses $contents where it is not stripped
-    s#^(.*?)\s*\\\s*$#$1#; # trailing '\' possibly preceded and/or followed by whitespace
+    # strip any other trailing '\' along with any whitespace around it such that it does not interfere with various matching below
+    my $trailing_backslash = s#^(.*?)\s*\\\s*$#$1#; # trailing '\' possibly preceded or followed by whitespace
+    my $essentially_blank_line = m/^\s*$/; # just whitespace and maybe a '\'
 
     # comments @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
@@ -529,8 +606,8 @@ while (<>) { # loop over all lines of all input files
 
     # detect end of comment, must be within multi-line comment, check if it is preceded by non-whitespace text
     if ((my ($head, $tail) = m|^(.*?)\*/(.*)$|) && $1 ne '/') { # ending comment: '*/'
-        report("neither space nor '*' before '*/'") if $head =~ m/[^*\s]$/;
-        report("missing space after '*/'") if $tail =~ m/^[^\s,;)}\]]/; # no space or ,;)}] after '*/'
+        report("missing space or '*' before '*/'") if $head =~ m/[^*\s]$/;
+        report("missing space (or ',', ';', ')', '}', ']') after '*/'") if $tail =~ m/^[^\s,;)}\]]/; # no space or ,;)}] after '*/'
         if (!($head =~ m|/\*|)) { # not begin of comment '/*', which is is handled below
             if ($in_comment == 0) {
                 report("unexpected '*/' outside comment");
@@ -555,7 +632,7 @@ while (<>) { # loop over all lines of all input files
     if (my ($head, $opt_minus, $tail) = m|^(.*?)/\*(-?)(.*)$|) { # begin of comment: '/*'
         report("missing space before '/*'")
             if $head =~ m/[^\s(\*]$/; # not space, '(', or or '*' (needed to allow '*/') before comment delimiter
-        report("missing space, '*' or '!' after '/*' or '/*-'") if $tail =~ m/^[^*\s!$self_test_exception]/;
+        report("missing space, '*', or '!' after '/*$opt_minus'") if $tail =~ m/^[^\s*!$self_test_exception]/;
         my $cmt_text = $opt_minus.$tail; # preliminary
         if ($in_comment > 0) {
             report("unexpected '/*' inside multi-line comment");
@@ -612,7 +689,39 @@ while (<>) { # loop over all lines of all input files
 
     # at this point all non-space portions of any types of comments have been blinded as @
 
-    goto LINE_FINISHED if m/^\s*$/; # essentially blank line: just whitespace (and maybe a trailing '\')
+    goto LINE_FINISHED if $essentially_blank_line;
+
+    # handle preprocessor directives @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
+
+    if (s/^(\s*#)(\s*)(\w+)//) { # line beginning with '#' and directive name;
+        # blank these portions to prevent confusion with C-level 'if', 'else', etc.
+        my ($lead, $space) = ($1, $2);
+        $preproc_directive = $3;
+        $_ = "$lead$space$preproc_directive$_" if $preproc_directive =~ m/^(define|include)$/; # yet do not blank #define or #include to prevent confusing the indentation or whitespace checks, resp.
+        $_ =  blind_nonspace($_) if $preproc_directive eq "error"; # blind error message
+        if ($in_preproc != 0) {
+            report("preprocessor directive within multi-line directive");
+            reset_indentation_state();
+        }
+        $in_preproc++;
+        report("indent = $count != 0 for '#'") if $count != 0;
+        report("'#$preproc_directive' with constant condition")
+            if $preproc_directive =~ m/^(if|elif)$/ && m/^[\W0-9]+$/ && !$trailing_backslash;
+        $preproc_if_nesting-- if $preproc_directive =~ m/^(else|elif|endif)$/;
+        if ($preproc_if_nesting < 0) {
+            $preproc_if_nesting = 0;
+            report("unexpected '#$preproc_directive' according to '#if' nesting");
+        }
+        my $space_count = length($space); # maybe could also use indentation before '#'
+        report("'#if' nesting indent = $space_count != $preproc_if_nesting") if $space_count != $preproc_if_nesting;
+        $preproc_if_nesting++ if $preproc_directive =~ m/^(if|ifdef|ifndef|else|elif)$/;
+        $ifdef__cplusplus = $preproc_directive eq "ifdef" && m/\s+__cplusplus\s*$/;
+
+        # handle indentation of preprocessor directive independently of surrounding normal code
+        $count = -1; # do not check indentation of first line of preprocessor directive
+        backup_indentation_state();
+        reset_indentation_state();
+    }
 
     # intra-line whitespace nits @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
@@ -662,10 +771,17 @@ while (<>) { # loop over all lines of all input files
         }
         # ignore paths in #include
         $intra_line =~ s/^(include\s*)(".*?"|<.*?>)/$1/e if $head =~ m/#/;
+        report("missing space before '$2'")
+            if $intra_line =~ m/(\S)((<<|>>)=)/ # '<<=' or >>=' without preceding space
+            || ($intra_line =~ m/(\S)([\+\-\*\/\/%\&\|\^\!<>=]=)/
+                && "$1$2" ne "<<=" && "$1$2" ne ">>=") # other <op>= or (in)equality without preceding space
+            || ($intra_line =~ m/(\S)=/
+                && !($1 =~ m/[\+\-\*\/\/%\&\|\^\!<>=]/)
+                && $intra_line =~ m/(\S)(=)/); # otherwise, '=' without preceding space
         # treat op= and comparison operators as simple '=', simplifying matching below
-        $intra_line =~ s/([\+\-\*\/\/%\&\|\^\!<>=]|<<|>>)=/=/g;
+        $intra_line =~ s/(<<|>>|[\+\-\*\/\/%\&\|\^\!<>=])=/=/g;
         # treat (type) variables within macro, indicated by trailing '\', as 'int' simplifying matching below
-        $intra_line =~ s/[A-Z_]+/int/g if $contents =~ m/^(.*?)\s*\\\s*$/;
+        $intra_line =~ s/[A-Z_]+/int/g if $trailing_backslash;
         # treat double &&, ||, <<, and >> as single ones, simplifying matching below
         $intra_line =~ s/(&&|\|\||<<|>>)/substr($1, 0, 1)/eg;
         # remove blinded comments etc. directly after [{(
@@ -675,9 +791,12 @@ while (<>) { # loop over all lines of all input files
         # treat remaining blinded comments and string literal contents as (single) space during matching below
         $intra_line =~ s/@+/ /g;                     # note that extra SPC has already been handled above
         $intra_line =~ s/\s+$//;                     # strip any (resulting) space at EOL
-        $intra_line =~ s/(for\s*\([^;]*);;(\))/"$1$2"/eg; # strip trailing ';;' in for (;;)
-        $intra_line =~ s/(for\s*\([^;]+;[^;]+);(\))/"$1$2"/eg; # strip trailing ';' in for (;;)
-        $intra_line =~ s/(for\s*\();(;)/"$1$2"/eg;   # replace leading ';;' in for (;;) by ';'
+        # replace ';;' or '; ;' by ';' in "for (;;)" and in "for (...)" unless "..." contains just SPC and ';' characters:
+        $intra_line =~ s/((^|\W)for\s*\()([^;]*?)(\s*)(;\s?);(\s*)([^;]*)(\))/
+          "$1$3$4".("$3$4$5$6$7" eq ";" || $3 ne "" || $7 ne "" ? "" : $5).";$6$7$8"/eg;
+        # strip trailing ';' or '; ' in "for (...)" except in "for (;;)" or "for (;; )":
+        $intra_line =~ s/((^|\W)for\s*\()([^;]*(;[^;]*)?)(;\s?)(\))/
+          "$1$3".($3 eq ";" ? $5 : "")."$6"/eg;
         $intra_line =~ s/(=\s*)\{ /"$1@ "/eg;        # do not report {SPC in initializers such as ' = { 0, };'
         $intra_line =~ s/, \};/, @;/g;               # do not report SPC} in initializers such as ' = { 0, };'
         report("space before '$1'") if $intra_line =~ m/[\w)\]]\s+(\+\+|--)/;  # postfix ++/-- with preceding space
@@ -686,11 +805,11 @@ while (<>) { # loop over all lines of all input files
         report("space before '$1'") if $intra_line =~ m/\s(\.|->)/;            # '.' or '->' with preceding space
         report("space after '$1'")  if $intra_line =~ m/(\.|->)\s/;            # '.' or '->' with following space
         $intra_line =~ s/\-\>|\+\+|\-\-/@/g;         # blind '->,', '++', and '--'
-        report("space before '$1'")     if $intra_line =~ m/[^:)]\s+(;)/;      # space before ';' but not after ':' or ')'
+        report("space before '$1'")     if $intra_line =~ m/[^:)]\s+(;)/;      # space before ';' but not after ':' or ')' # note that
+        # exceptions for "for (;; )" are handled above
         report("space before '$1'")     if $intra_line =~ m/\s([,)\]])/;       # space before ,)]
         report("space after '$1'")      if $intra_line =~ m/([(\[~!])\s/;      # space after ([~!
         report("space after '$1'")      if $intra_line =~ m/(defined)\s/;      # space after 'defined'
-        report("missing space before '=' or '<op>='") if $intra_line =~ m/\S(=)/;   # '=' etc. without preceding space
         report("missing space before '$1'")  if $intra_line =~ m/\S([|\/%<>^\?])/;  # |/%<>^? without preceding space
         # TODO ternary ':' without preceding SPC, while allowing no SPC before ':' after 'case'
         report("missing space before binary '$2'")  if $intra_line =~ m/([^\s{()\[e])([+\-])/; # '+'/'-' without preceding space or {()[e
@@ -710,33 +829,11 @@ while (<>) { # loop over all lines of all input files
         report("space after function/macro name")
                                       if $intra_line =~ m/(\w+)\s+\(/        # fn/macro name with space before '('
        && !($1 =~ m/^(sizeof|if|else|while|do|for|switch|case|default|break|continue|goto|return|void|char|signed|unsigned|int|short|long|float|double|typedef|enum|struct|union|auto|extern|static|const|volatile|register)$/) # not keyword
-                                    && !(m/^\s*#\s*define\s/); # we skip macro definitions here because macros
-                                    # without parameters but with body beginning with '(', e.g., '#define X (1)',
-                                    # would lead to false positives - TODO also check for macros with parameters
+                                    && !(m/^\s*#\s*define\s+\w+\s+\(/); # not a macro without parameters having a body that starts with '('
         report("missing space before '{'")   if $intra_line =~ m/[^\s{(\[]\{/;      # '{' without preceding space or {([
         report("missing space after '}'")    if $intra_line =~ m/\}[^\s,;\])}]/;    # '}' without following space or ,;])}
     }
 
-    # preprocessor directives @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-
-    # handle preprocessor directives
-    if (m/^\s*#(\s*)(\w+)/) { # line beginning with '#'
-        my $space_count = length($1); # maybe could also use indentation before '#'
-        my $directive = $2;
-        report("indent = $count != 0 for '#'") if $count != 0;
-        $directive_nesting-- if $directive =~ m/^(else|elif|endif)$/;
-        if ($directive_nesting < 0) {
-            $directive_nesting = 0;
-            report("unexpected '#$directive'");
-        }
-        report("'#' directive nesting = $space_count != $directive_nesting") if $space_count != $directive_nesting;
-        $directive_nesting++ if $directive =~ m/^if|ifdef|ifndef|else|elif$/;
-        $ifdef__cplusplus = m/^\s*#\s*ifdef\s+__cplusplus\s*$/;
-        goto POSTPROCESS_DIRECTIVE unless $directive =~ m/^define$/; # skip normal code handling except for #define
-        # TODO improve handling of indents of preprocessor directives ('\', $in_directive != 0) vs. normal C code
-        $count = -1; # do not check indentation of #define
-    }
-
     # adapt required indentation @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
     s/(\w*ASN1_[A-Z_]+END\w*([^(]|\(.*?\)|$))/$1;/g; # treat *ASN1_*END*(..) macro calls as if followed by ';'
@@ -807,7 +904,7 @@ while (<>) { # loop over all lines of all input files
         # handle opening brace '{' after if/else/while/for/switch/do on line before
         if ($hanging_offset > 0 && m/^[\s@]*{/ && # leading opening '{'
             $line_before > 0 &&
-            $contents_before_ =~ m/(^|^.*\W)(if|else|while|for|switch|do)(\W.*$|$)/) {
+            $contents_before_ =~ m/(^|^.*\W)(if|else|while|for|(OSSL_)?LIST_FOREACH(_\w+)?|switch|do)(\W.*$|$)/) {
             $keyword_opening_brace = $1;
             $hanging_offset -= INDENT_LEVEL; # cancel newly hanging_offset
         }
@@ -825,30 +922,30 @@ while (<>) { # loop over all lines of all input files
     }
 
     # potential adaptations of indent in first line of macro body in multi-line macro definition
-    if ($in_directive > 0 && $in_macro_header > 0) {
+    if ($in_preproc != 0 && $in_macro_header > 0) {
         if ($in_macro_header > 1) { # still in macro definition header
             $in_macro_header += parens_balance($_);
         } else { # begin of macro body
             $in_macro_header = 0;
-            if ($count == $block_indent - $directive_offset # body began with same indentation as preceding code
+            if ($count == $block_indent - $preproc_offset # body began with same indentation as preceding code
                 && $sloppy_macro) { # workaround for this situation is enabled
-                $block_indent -= $directive_offset;
-                $directive_offset = 0;
+                $block_indent -= $preproc_offset;
+                $preproc_offset = 0;
             }
         }
     }
 
     # check required indentation @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
-    check_indent() if $count >= 0; # not for #define and not if multi-line string literal is continued
+    check_indent() if $count >= 0; # not for start of preprocessor directive and not if multi-line string literal is continued
 
     # check for blank lines within/after local decls @@@@@@@@@@@@@@@@@@@@@@@@@@@
 
     if ($in_block_decls >= 0 &&
-        $in_comment == 0 && !m/^\s*\*?@/ && # not in multi-line comment nor an intra-line comment
+        $in_comment == 0 && !m/^\s*\*?@/ && # not in a multi-line or intra-line comment
         !$in_expr && $expr_indent == 0 && $in_typedecl == 0) {
-        my $blank_line_before = $line > 1
-            && $code_contents_before =~ m/^\s*(\\\s*)?$/; # essentially blank line: just whitespace (and maybe a trailing '\')
+        my $blank_line_before = $line > 1 && $code_contents_before =~ m/^\s*(\\\s*)?$/;
+        # essentially blank line before: just whitespace and maybe a '\'
         if (m/^[\s(]*(char|signed|unsigned|int|short|long|float|double|enum|struct|union|auto|extern|static|const|volatile|register)(\W|$)/ # clear start of local decl
             || (m/^(\s*(\w+|\[\]|[\*()]))+?\s+[\*\(]*\w+(\s*(\)|\[[^\]]*\]))*\s*[;,=]/ # weak check for decl involving user-defined type
                 && !m/^\s*(\}|sizeof|if|else|while|do|for|switch|case|default|break|continue|goto|return)(\W|$)/)) {
@@ -867,17 +964,20 @@ while (<>) { # loop over all lines of all input files
 
     # do some further checks @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
-    my $outermost_level = $block_indent == 0 + ($in_directive > 0 ? $directive_offset : 0);
+    my $outermost_level = $block_indent - $preproc_offset == 0;
 
-    report("more than one stmt") if !m/(^|\W)for(\W.*|$)/ && # no 'for' - TODO improve matching
+    report("more than one stmt") if !m/(^|\W)(for|(OSSL_)?LIST_FOREACH(_\w+)?)(\W.*|$)/ && # no 'for' - TODO improve matching
         m/;.*;/; # two or more terminators ';', so more than one statement
 
     # check for code block containing a single line/statement
     if ($line_before2 > 0 && !$outermost_level && # within function body
         $in_typedecl == 0 && @nested_indents == 0 && # neither within type declaration nor inside stmt/expr
-        m/^[\s@]*\}/) { # leading closing brace '}', any preceding blinded comment must not be matched
+        m/^[\s@]*\}\s*(\w*)/) { # leading closing brace '}', any preceding blinded comment must not be matched
         # TODO extend detection from single-line to potentially multi-line statement
+        my $next_word = $1;
         if ($line_opening_brace > 0 &&
+            ($keyword_opening_brace ne "if" ||
+             $extended_1_stmt || $next_word ne "else") &&
             ($line_opening_brace == $line_before2 ||
              $line_opening_brace == $line_before)
             && $contents_before =~ m/;/) { # there is at least one terminator ';', so there is some stmt
@@ -893,8 +993,6 @@ while (<>) { # loop over all lines of all input files
         if (m/(['"]|([\+\-\*\/\/%\&\|\^<>]\s*)?\W[0-9]+L?|\WNULL)\s*([\!<>=]=|[<=>])([<>]?)/ &&
             $2 eq "" && (($3 ne "<" && $3 ne "='" && $3 ne ">") || $4 eq ""));
 
-    # TODO report #if 0 and #if 1
-
     # TODO report needless use of parentheses, while
     #      macro parameters should always be in parens (except when passed on), e.g., '#define ID(x) (x)'
 
@@ -906,7 +1004,7 @@ while (<>) { # loop over all lines of all input files
     my $assignment_start = 0;
     my $tmp = $_;
     $tmp =~ s/[\!<>=]=/@@/g; # blind (in-)equality symbols like '<=' as '@@' to prevent matching them as '=' below
-    if      (m/^((^|.*\W)(if|while|for|switch))(\W.*|$)$/) { # (last) if/for/while/switch
+    if      (m/^((^|.*\W)(if|while|for|(OSSL_)?LIST_FOREACH(_\w+)?|switch))(\W.*|$)$/) { # (last) if/for/while/switch
         $paren_expr_start = 1;
     } elsif (m/^((^|.*\W)(return|enum))(\W.*|$)/             # (last) return/enum
         && !$in_expr && @nested_indents == 0 && parens_balance($1) == 0) { # not nested enum
@@ -936,7 +1034,7 @@ while (<>) { # loop over all lines of all input files
     }
 
     # set $hanging_offset and $keyword_opening_brace for do/else
-    if (my ($head, $mid, $tail) = m/(^|^.*\W)(else|do)(\W.*|$)$/) { # last else/do, where 'do' is preferred
+    if (my ($head, $mid, $tail) = m/(^|^.*\W)(else|do)(\W.*|$)$/) { # last else/do, where 'do' is preferred, but not #else
         my $code_before = $head =~ m/[^\s\@}]/; # leading non-whitespace non-comment non-'}'
         report("code before '$mid'") if $code_before;
         report("code after '$mid'" ) if $tail =~ m/[^\s\@{]/# trailing non-whitespace non-comment non-'{' (non-'\')
@@ -971,10 +1069,10 @@ while (<>) { # loop over all lines of all input files
         $hanging_offset += INDENT_LEVEL if m/\*.*\(/; # '*' followed by '(' - seems consistent with Emacs C mode
     }
 
-    my $bak_in_expr = $in_expr;
+    my $local_in_expr = $in_expr;
     my $terminator_position = update_nested_indents($_, $nested_indents_position);
 
-    if ($bak_in_expr) {
+    if ($local_in_expr) {
         # on end of non-if/while/for/switch (multi-line) expression (i.e., return/enum/assignment) and
         # on end of statement/type declaration/variable definition/function header
         if ($terminator_position >= 0 && ($in_typedecl == 0 || @nested_indents == 0)) {
@@ -1022,24 +1120,24 @@ while (<>) { # loop over all lines of all input files
     }
 
     # remember line number and header containing name of last function defined for reports w.r.t. MAX_BODY_LENGTH
-    if ($outermost_level && m/(\w+)\s*\(/ && $1 ne "STACK_OF") {
+    if ($in_preproc == 0 && $outermost_level && m/(\w+)\s*\(/ && $1 ne "STACK_OF") {
         $line_function_start = $line;
         $last_function_header = $contents;
     }
 
     # special checks for last, typically trailing opening brace '{' in line
     if (my ($head, $tail) = m/^(.*)\{(.*)$/) { # match last ... '{'
-        if ($in_directive == 0 && !$in_expr && $in_typedecl == 0) {
+        if (!$in_expr && $in_typedecl == 0) {
             if ($outermost_level) {
-                if (!$assignment_start && !$bak_in_expr) {
+                if (!$assignment_start && !$local_in_expr) {
                     # at end of function definition header (or stmt or var definition)
-                    report("'{' not at beginning") if $head ne "";
-                    $line_body_start = $contents =~ m/LONG BODY/ ? 0 : $line;
+                    report("'{' not at line start") if length($head) != $preproc_offset && $head =~ m/\)\s*/; # at end of function definition header
+                    $line_body_start = $contents =~ m/LONG BODY/ ? 0 : $line if $line_function_start != 0;
                 }
             } else {
-                $line_opening_brace = $line if $keyword_opening_brace =~ m/do|while|for/;
+                $line_opening_brace = $line if $keyword_opening_brace =~ m/if|do|while|for|(OSSL_)?LIST_FOREACH(_\w+)?/;
                 # using, not assigning, $keyword_opening_brace here because it could be on an earlier line
-                $line_opening_brace = $line if $keyword_opening_brace =~ m/if|else/ && $extended_1_stmt &&
+                $line_opening_brace = $line if $keyword_opening_brace eq "else" && $extended_1_stmt &&
                 # TODO prevent false positives for if/else where braces around single-statement branches
                 # should be avoided but only if all branches have just single statements
                 # The following helps detecting the exception when handling multiple 'if ... else' branches:
@@ -1050,11 +1148,11 @@ while (<>) { # loop over all lines of all input files
         }
     }
 
-    # check for opening brace after if/while/for/switch/do not on same line
+    # check for opening brace after if/while/for/switch/do missing on same line
     # note that "missing '{' on same line after '} else'" is handled further below
     if (/^[\s@]*{/ && # leading '{'
         $line_before > 0 && !($contents_before_ =~ m/^\s*#/) && # not preprocessor directive '#if
-        (my ($head, $mid, $tail) = ($contents_before_ =~ m/(^|^.*\W)(if|while|for|switch|do)(\W.*$|$)/))) {
+        (my ($head, $mid, $tail) = ($contents_before_ =~ m/(^|^.*\W)(if|while|for|(OSSL_)?LIST_FOREACH(_\w+)?|switch|do)(\W.*$|$)/))) {
         my $brace_after  = $tail =~ /^[\s@]*{/; # any whitespace or comments then '{'
         report("'{' not on same line as preceding '$mid'") if !$brace_after;
     }
@@ -1098,37 +1196,33 @@ while (<>) { # loop over all lines of all input files
         }
     }
 
-  POSTPROCESS_DIRECTIVE:
     # on begin of multi-line preprocessor directive, adapt indent
-    # need to use original line contents because trailing '\' may have been stripped above
-    if ($contents =~ m/^(.*?)[\s@]*\\[\s@]*$/) { # trailing '\' (which is not stripped from $contents),
-        # typically used in macro definitions (or other preprocessor directives)
-        if ($in_directive == 0) {
+    if ($in_comment == 0 && $trailing_backslash) {
+        # trailing '\'typically used in preprocessor directive like '#define'
+        if ($in_preproc == 1) { # start of multi-line preprocessor directive
+            # note that backup+reset_indentation_state() has already been called
             $in_macro_header = m/^\s*#\s*define(\W|$)?(.*)/ ? 1 + parens_balance($2) : 0; # '#define' is beginning
-            $directive_offset = INDENT_LEVEL;
-            $block_indent += $directive_offset;
+            $preproc_offset = INDENT_LEVEL;
+            $block_indent = $preproc_offset;
         }
-        $in_directive += 1;
+        $in_preproc += 1;
     }
 
     # post-processing at end of line @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
   LINE_FINISHED:
     $code_contents_before = $contents if
-        !m/^\s*#(\s*)(\w+)/ && # not single-line directive
-        $in_comment == 0 && !m/^\s*\*?@/; # not in multi-line comment nor an intra-line comment
+        !m/^\s*#(\s*)(\w+)/ && # not single-line preprocessor directive
+        $in_comment == 0 && !m/^\s*\*?@/; # not in a multi-line comment nor in an intra-line comment
 
-    # on end of multi-line preprocessor directive, adapt indent
-    if ($in_directive > 0 &&
-        # need to use original line contents because trailing \ may have been stripped
-        !($contents =~ m/^(.*?)[\s@]*\\[\s@]*$/)) { # no trailing '\'
-        $block_indent -= $directive_offset;
-        $in_directive = 0;
-        # macro body typically does not include terminating ';'
-        $hanging_offset = 0; # compensate for this in case macro ends, e.g., as 'while (0)'
+    # on end of (possibly multi-line) preprocessor directive, adapt indent
+    if ($in_preproc != 0 && !$trailing_backslash) { # no trailing '\'
+        $in_preproc = 0;
+        $preproc_offset = 0;
+        restore_indentation_state();
     }
 
-    if (m/^\s*$/) { # at begin of file essentially blank line: just whitespace (and maybe a '\')
+    if ($essentially_blank_line) {
             report("leading ".($1 eq "" ? "blank" :"whitespace")." line") if $line == 1 && !$sloppy_SPC;
     } else {
         if ($line_before > 0) {
@@ -1166,7 +1260,7 @@ while (<>) { # loop over all lines of all input files
         report_flexibly($line, +@nested_block_indents." unclosed '{'", "(EOF)\n") if @nested_block_indents != 0;
 
         # sanity-check balance of #if ... #endif via final preprocessor directive indent at end of file
-        report_flexibly($line, "$directive_nesting unclosed '#if'", "(EOF)\n") if $directive_nesting != 0;
+        report_flexibly($line, "$preproc_if_nesting unclosed '#if'", "(EOF)\n") if $preproc_if_nesting != 0;
 
         reset_file_state();
     }
@@ -1176,6 +1270,6 @@ while (<>) { # loop over all lines of all input files
 
 my $num_other_reports = $num_reports - $num_indent_reports - $num_nesting_issues
     - $num_syntax_issues - $num_SPC_reports - $num_length_reports;
-print "$num_reports ($num_indent_reports indentation, $num_nesting_issues directive nesting, ".
+print "$num_reports ($num_indent_reports indentation, $num_nesting_issues '#if' nesting indent, ".
     "$num_syntax_issues syntax, $num_SPC_reports whitespace, $num_length_reports length, $num_other_reports other)".
     " issues have been found by $0\n" if $num_reports != 0 && !$self_test;
